<?php

/* Copyright (C) 2024 SuperAdmin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/**
 * \file    core/triggers/interface_99_modEcotax_EcotaxTriggers.class.php
 * \ingroup ecotax
 * \brief   Example trigger.
 *
 * Put detailed description here.
 *
 * \remarks You can create other triggers by copying this one.
 * - File name should be either:
 *      - interface_99_modEcotax_MyTrigger.class.php
 *      - interface_99_all_MyTrigger.class.php
 * - The file must stay in core/triggers
 * - The class name must be InterfaceMytrigger
 */
require_once DOL_DOCUMENT_ROOT . '/core/triggers/dolibarrtriggers.class.php';
require_once DOL_DOCUMENT_ROOT . '/product/class/product.class.php';
require_once DOL_DOCUMENT_ROOT.'/core/class/cunits.class.php';
dol_include_once('/ecotax/lib/ecotax.lib.php');

/**
 *  Class of triggers for Ecotax module
 */
class InterfaceEcotaxTriggers extends DolibarrTriggers
{

	static $ecotaxes_ids = array();

	/**
	 * Constructor
	 *
	 * @param DoliDB $db Database handler
	 */
	public function __construct($db)
	{
		$this->db = $db;

		$this->name = preg_replace('/^Interface/i', '', get_class($this));
		$this->family = "demo";
		$this->description = "Ecotax triggers.";
		// 'development', 'experimental', 'dolibarr' or version
		$this->version = 'development';
		$this->picto = 'ecotax@ecotax';
	}

	/**
	 * Trigger name
	 *
	 * @return string Name of trigger file
	 */
	public function getName()
	{
		return $this->name;
	}

	/**
	 * Trigger description
	 *
	 * @return string Description of trigger file
	 */
	public function getDesc()
	{
		return $this->description;
	}

	/**
	 * Function called when a Dolibarrr business event is done.
	 * All functions "runTrigger" are triggered if file
	 * is inside directory core/triggers
	 *
	 * @param string 		$action 	Event action code
	 * @param CommonObject 	$object 	Object
	 * @param User 			$user 		Object user
	 * @param Translate 	$langs 		Object langs
	 * @param Conf 			$conf 		Object conf
	 * @return int              		<0 if KO, 0 if no triggered ran, >0 if OK
	 */
	public function runTrigger($action, $object, User $user, Translate $langs, Conf $conf)
	{
		if (!isModEnabled('ecotax')) {
			return 0; // If module is not enabled, we do nothing
		}

		// You can isolate code for each action in a separate method: this method should be named like the trigger in camelCase.
		// For example : COMPANY_CREATE => public function companyCreate($object, User $user, Translate $langs, Conf $conf)
		$methodName = lcfirst(str_replace(' ', '', ucwords(str_replace('_', ' ', strtolower($action)))));
		$callback = array($this, $methodName);
		if (is_callable($callback)) {
			dol_syslog("Trigger '".$this->name."' for action '$action' launched by ".__FILE__.". id=".$object->id);
			return call_user_func($callback, $object, $user, $langs, $conf);
		}

		$langs->load('ecotax@ecotax');

		switch ($action) {
			case 'LINEPROPAL_INSERT':
			case 'LINEORDER_INSERT':
			case 'LINEBILL_INSERT':
			case 'LINEBILLREC_INSERT': // Does not exist
			case 'LINESUPPLIERBILLREC_INSERT': // Does not exist
			case 'LINEBILL_SUPPLIER_CREATE':
			case 'LINEORDER_SUPPLIER_CREATE':
				if ($this->shouldHandleEcotax($object)) {
					$savconf = getDolGlobalInt('SUPPLIER_ORDER_WITH_PREDEFINED_PRICES_ONLY');
					$conf->global->SUPPLIER_ORDER_WITH_PREDEFINED_PRICES_ONLY = 0;
					$res = $this->insertEcotax($object, $langs);
					$conf->global->SUPPLIER_ORDER_WITH_PREDEFINED_PRICES_ONLY = $savconf;
					return $res;
				}
				break;
			case 'LINEPROPAL_UPDATE':
			case 'LINEPROPAL_MODIFY':
			case 'LINEORDER_UPDATE':
			case 'LINEORDER_MODIFY':
			case 'LINEBILL_UPDATE':
			case 'LINEBILL_MODIFY':
			case 'LINEBILLREC_UPDATE':
			case 'LINEBILLREC_MODIFY':
			case 'LINESUPPLIERBILLREC_MODIFY':
			case 'LINESUPPLIERBILLREC_UPDATE':
			case 'LINEORDER_SUPPLIER_UPDATE':
			case 'LINEORDER_SUPPLIER_MODIFY':
			case 'LINEBILL_SUPPLIER_UPDATE':
			case 'LINEBILL_SUPPLIER_MODIFY':
				if ($this->shouldHandleEcotax($object)) {
					return $this->updateEcotax($object, $langs);
				}
				break;
			case 'LINEPROPAL_DELETE':
			case 'LINEORDER_DELETE':
			case 'LINEBILL_DELETE':
			case 'LINEBILLREC_DELETE':
			case 'LINESUPPLIERBILLREC_DELETE':
			case 'LINEBILL_SUPPLIER_DELETE':
			case 'LINEORDER_SUPPLIER_DELETE':
				if ($this->shouldHandleEcotax($object)) {
					return $this->deleteEcotax($object, $langs);
				}
				break;
			default:
				return 0;
		}

		return 0;
	}

	public function productModify(Product $object, User $user, Translate $langs, Conf $conf)
	{
		if ($this->shouldInsertProductExtraEcotaxAmount($object)) {
			return $this->insertProductExtraEcotaxAmount($object);
		}

		if ($this->shouldVerifyEcotaxType($object)) {
			return $this->verifyEcotaxType($object);
		}

		return 0;
	}

	protected function shouldInsertProductExtraEcotaxAmount($object)
	{
		if ($object->element != 'product') {
			return false;
		}

		if (!GETPOSTISSET('action') || GETPOST('action') != 'update_extras') {
			return false;
		}

		if (!GETPOSTISSET('attribute') || GETPOST('attribute') != 'ecotax_unit_amount') {
			return false;
		}

		if (!GETPOSTISSET('options_ecotax_unit_amount') || empty(GETPOST('options_ecotax_unit_amount'))) {
			return false;
		}

		return true;
	}

	/**
	 * As 'options_ecotax_unit_amount' is hidden by default, dolibarr don't update it with other extrafields.
	 * @param type $object
	 * @param type $langs
	 * @return int
	 */
	protected function insertProductExtraEcotaxAmount($object)
	{
		$object->array_options['options_ecotax_unit_amount'] = price2num(GETPOST('options_ecotax_unit_amount'));

		$retInsert = $object->insertExtraFields();

		if ($retInsert < 0) {
			$this->error = $object->error;
			$this->errors += $object->errors;
			return $retInsert;
		}

		return 0;
	}

	protected function shouldVerifyEcotaxType($object)
	{
		if ($object->element != 'product') {
			return false;
		}

		if (!GETPOSTISSET('action')) {
			return false;
		} elseif (GETPOST('action') == 'update_extras' && (!GETPOSTISSET('attribute') || GETPOST('attribute') != 'ecotax_ecotax')) {
			return false;
		} elseif (GETPOST('action') == 'update' && (!GETPOSTISSET('weight') || !empty(GETPOST('weight')))) {
			return false;
		} elseif (GETPOST('action') != 'update_extras' && GETPOST('action') != 'update') {
			return false;
		}

		if (!GETPOSTISSET('options_ecotax_ecotax') || empty(GETPOST('options_ecotax_ecotax'))) {
			return false;
		}

		return true;
	}

	/**
	 * As 'options_ecotax_unit_amount' is hidden by default, dolibarr don't update it with other extrafields.
	 * @param type $object
	 * @param type $langs
	 * @return int
	 */
	protected function verifyEcotaxType($object)
	{
		global $langs;

		$ecotax = GETPOST('options_ecotax_ecotax');
		$sql = '
			SELECT rowid, code, fk_service, fk_unit, unit_amount, subject_to_vat
			FROM ' . $this->db->prefix() . 'c_ecotax_type
			WHERE rowid = ' . intval($ecotax) . '
			LIMIT 1';

		$resql = $this->db->query($sql);

		if (!$resql) {
			$this->error = $this->db->lasterror();
			throw new Exception("", 1);
		}

		$ecotax_type = $this->db->fetch_object($resql);

		if ($ecotax_type->fk_unit > 0 && empty($object->weight)) {
			$this->error = $langs->trans('ProductLinkToEcotaxButDoesnotHaveWeight', $ecotax_type->code);
			return -1;
		}

		return 0;
	}

	/**
	 * Function to determine if ecotax behavior should be applied here
	 * @param type $object
	 * @return bool
	 */
	protected function shouldHandleEcotax($object)
	{

		if ($object->fk_product_type) {
			return false;
		}

		$handledLines = ['propaldet', 'commandedet', 'facturedet', 'facturerecdet', 'facture_fourn_det', 'commande_fournisseurdet'];
		if (!in_array($object->element, $handledLines)) {
			return false;
		}

		if (!empty($object->context['createfromclone']) && !getDolGlobalInt('ECOTAX_ALSO_ON_CREATE_FROM_CLONE')) {
			return false;
		}

		if (!empty($object->origin) && !getDolGlobalInt('ECOTAX_ALSO_ON_LINK_ORIGIN')) {
			return false;
		}

		if (empty($object->fk_product)) {
			return false;
		}

		if ( $object->fk_product_type == 1 || $object->product_type == 1 /*|| (isset($object->type) && $object->type == 1 )*/) { // $line->type seems not to exist.
			return false;
		}

		return true;
	}

	/**
	 * When a product which have ecotax defined in its extrafield is added to a document this function add also the 
	 * corresponding ecotax service
	 * @param type $object
	 * @param type $langs
	 * @return int
	 */
	protected function insertEcotax($object, $langs)
	{
		try {
			[$parent, $ecotax_service, $product, $ecotax_type, $scale] = $this->prepareObjects($object, $langs);
		} catch (Exception $e) {
			return $e->getCode();
		}

		// If ecotax_type does not define a unit_amount we need to recover amount set on product extrafield
		$unit_amount = $ecotax_type->unit_amount;
		if (empty($unit_amount)) {
			$unit_amount = $product->array_options['options_ecotax_unit_amount'];
		}

		// If ecotax_type  is not subjet to vat we set it to 0
		$tva_tx = floatval($object->tva_tx);

		if ($ecotax_type->subject_to_vat < 0) {
			$tva_tx = 0;
		}

		$qty = $object->qty;
		if ($ecotax_type->fk_unit > 0) {
			$qty = ($qty * $product->weight) * 10**$scale;
		}

		// Some fields are set twice to handle difference in addline function definition of different documents
		$newLine = [
			'desc'			 => $ecotax_type->code,
			'pu_ht'			 => $unit_amount,
			'pu'			 => $unit_amount,
			'qty'			 => $qty,
			'txtva'			 => $tva_tx,
			'fk_product'	 => $ecotax_service->id,
			// 'notrigger'		 => true,
			'txlocaltax1'	 => $tva_tx,
			'txlocaltax2'	 => $tva_tx,
			'fk_parent_line' => $object->id,
			'special_code'	 => $object->id,
		];

		$ret = $this->_simple_method_call($parent, 'addline', $newLine);
		if ($ret < 0) {
			$this->errors = array_merge($this->errors, [$parent->error], $parent->errors);
			return $ret;
		} elseif ($ret > 0) {
			// check that the line has not been deleted inbetween
			$sql = "SELECT rowid FROM ".$this->db->prefix().$object->table_element." WHERE rowid = ".(int) $ret;
			$resql = $this->db->query($sql);
			if ($this->db->num_rows($resql) > 0) {
				setEventMessage($langs->transnoentities('EcotaxLineWasAdded', $ecotax_type->code, $product->label));
			};
		}

		// $unit;
		return 0;
	}

	protected function updateEcotax($object, $langs)
	{
		try {
			[$parent, $ecotax_service, $product, $ecotax_type, $scale] = $this->prepareObjects($object, $langs);
		} catch (Exception $e) {
			return $e->getCode();
		}

		// Object is the parent line, we need to recover ecotax line in the document.
		// For that fk_parent_line have been set when ecotax line was inserted.
		// Unfortunately order_supplier does not have parent_line field so we need to set special code also
		$sql = '
			SELECT rowid
			FROM ' . $this->db->prefix() . $object->table_element . '
			WHERE fk_parent_line=' . $object->id . ' OR special_code=' . $object->id . '
		';

		$resql = $this->db->query($sql);

		if (!$resql) {
			$this->error = $this->db->lasterror();
			return -1;
		}

		$obj = $this->db->fetch_object($resql);

		if (empty($obj->rowid)) {
			return 0;
		}

		// If ecotax_type does not define a unit_amount we need to recover amount set on product extrafield
		$unit_amount = $ecotax_type->unit_amount;
		if (empty($unit_amount)) {
			$unit_amount = $product->array_options['options_ecotax_unit_amount'];
		}

		// If ecotax_type  is not subjet to vat we set it to 0
		$tva_tx = $object->tva_tx;
		if ($ecotax_type->subject_to_vat < 0) {
			$tva_tx = 0;
		}

		$qty = $object->qty;
		if ($ecotax_type->fk_unit > 0) {
			$qty = ($qty * $product->weight) * 10**$scale;
		}

		// Some fields are set twice to handle difference in updateline function definittion of different documents
		$updatedLine = [
			'rowid'			 => $obj->rowid,
			'id'			 => $obj->rowid,
			'desc'			 => $ecotax_type->code,
			'pu_ht'			 => $unit_amount,
			'pu'			 => $unit_amount,
			'qty'			 => $qty,
			'txtva'			 => $tva_tx,
			'fk_product'	 => $ecotax_service->id,
			// 'notrigger'		 => true,
			'txlocaltax1'	 => $tva_tx,
			'txlocaltax2'	 => $tva_tx,
			'remise_percent' => 0,
			'fk_parent_line' => $object->id,
			'special_code'	 => $object->id,
			'vatrate'	 	 => $tva_tx,
		];

		$ret = $this->_simple_method_call($parent, 'updateline', $updatedLine);
		if ($ret < 0) {
			return $ret;
		}

		// $unit;
		return 0;
	}

	protected function deleteEcotax($object, $langs)
	{
		global $user;

		try {
			[$parent, $ecotax_service, $product, $ecotax_type] = $this->prepareObjects($object, $langs);
		} catch (Exception $e) {
			return $e->getCode();
		}

		// Object is the parent line, we need to recover ecotax line in the document.
		// For that fk_parent_line have been set when ecotax line was inserted.
		// Unfortunately order_supplier does not have parent_line field so we need to set special code also
		$sql = '
			SELECT rowid
			FROM ' . $this->db->prefix() . $object->table_element . '
			WHERE fk_parent_line=' . $object->id . ' OR special_code=' . $object->id . '
		';

		$resql = $this->db->query($sql);

		if (!$resql) {
			$this->error = $this->db->lasterror();
			return -1;
		}

		$obj = $this->db->fetch_object($resql);

		if (empty($obj->rowid)) {
			return 0;
		}

		$newLine = [
			'rowid'	 => $obj->rowid,
			'lineid' => $obj->rowid,
			'idline' => $obj->rowid,
			'id'	 => $parent->id,
			'user'	 => $user,
		];

		$ret = $this->_simple_method_call($parent, 'deleteline', $newLine);
		if ($ret < 0) {
			return $ret;
		}

		setEventMessage($langs->transnoentities('EcotaxLineWasDeleted', $ecotax_type->code, $product->label), 'errors');

		return 0;
	}

	/**
	 * This function load object than are used to add update and delete line.
	 * That is:
	 * - Parent object (order, invoice, ...)
	 * - Ecotax service link to this product if exist
	 * - Product of current line
	 * - Ecotax dict entry
	 * @param type $object
	 * @param type $langs
	 * @return type
	 * @throws Exception
	 */
	private function prepareObjects($object, $langs)
	{
		$product = new Product($this->db);
		$retFetch = $product->fetch($object->fk_product);

		if ($retFetch < 0) {
			$this->error = $product->error;
			$this->errors += $product->errors;
			throw new Exception("", $retFetch);
		}

		if (empty($retFetch)) {
			$this->error = $langs->trans("ProductNotFound");
			throw new Exception("", $retFetch);
		}

		if (empty($product->array_options['options_ecotax_ecotax'])) {
			throw new Exception("", 0);
		}

		$parent = $this->findParent($object);

		$sql = '
			SELECT rowid, code, fk_service, fk_unit, unit_amount, subject_to_vat
			FROM ' . $this->db->prefix() . 'c_ecotax_type
			WHERE rowid = ' . intval($product->array_options['options_ecotax_ecotax']) . '
			LIMIT 1';

		$resql = $this->db->query($sql);

		if (!$resql) {
			$this->error = $this->db->lasterror();
			throw new Exception("", 1);
		}

		$ecotax_type = $this->db->fetch_object($resql);

		$ecotax_service = new Product($this->db);

		$retFetchService = $ecotax_service->fetch($ecotax_type->fk_service);

		if ($retFetchService < 0) {
			$this->error = $ecotax_service->error;
			$this->errors += $ecotax_service->errors;
			throw new Exception("", $retFetchService);
		}

		if (empty($retFetchService)) {
			$this->error = $langs->trans('ErrorEcoTaxServiceNotFound', $ecotax_type->code);
			throw new Exception("", -1);
		}

		$scale = 0;
		if ($ecotax_type->fk_unit > 0) {
			if (empty($product->weight)) {
				$this->error = $langs->trans('ProductLinkToEcotaxButDoesnotHaveWeight', $ecotax_type->code);
				throw new Exception("", -1);
			}

			$measuringUnitsEcotax = new CUnits($this->db);

			$retFetchEcotaxUnit = $measuringUnitsEcotax->fetch($ecotax_type->fk_unit);

			if ($retFetchEcotaxUnit < 0) {
				$this->error = $measuringUnitsEcotax->error;
				$this->errors += $measuringUnitsEcotax->errors;
				throw new Exception("", $retFetchEcotaxUnit);
			}

			if ($measuringUnitsEcotax->scale != $product->weight_units) {
				$scale = $product->weight_units - $measuringUnitsEcotax->scale;
			}
		}

		return [$parent, $ecotax_service, $product, $ecotax_type, $scale];
	}

	/**
	 * Load parent object commande/propal/etc...
	 * @global type $langs
	 * @param type $object
	 * @return \CommandeFournisseur
	 * @throws Exception
	 */
	private function findParent($object)
	{
		global $langs;

		switch ($object->element) {
			case 'propaldet':
				require_once DOL_DOCUMENT_ROOT . '/comm/propal/class/propal.class.php';
				$parent = new Propal($this->db);
				$retFetch = $parent->fetch($object->fk_propal);
				break;
			case 'commandedet':
				require_once DOL_DOCUMENT_ROOT . '/commande/class/commande.class.php';
				$parent = new Commande($this->db);
				$retFetch = $parent->fetch($object->fk_commande);
				break;
			case 'facturedet':
				require_once DOL_DOCUMENT_ROOT . '/compta/facture/class/facture.class.php';
				$parent = new Facture($this->db);
				$retFetch = $parent->fetch($object->fk_facture);
				break;
			case 'facture_fourn_det':
				require_once DOL_DOCUMENT_ROOT . '/fourn/class/fournisseur.facture.class.php';
				$parent = new FactureFournisseur($this->db);
				$retFetch = $parent->fetch($object->fk_facture_fourn);
				break;
			case 'commande_fournisseurdet':
				require_once DOL_DOCUMENT_ROOT . '/fourn/class/fournisseur.commande.class.php';
				$parent = new CommandeFournisseur($this->db);
				$retFetch = $parent->fetch($object->fk_commande);
				break;
		}

		if ($retFetch < 0) {
			$this->error = $parent->error;
			$parent->errors += $parent->errors;
			throw new Exception("", $retFetch);
		}

		if (empty($retFetch)) {
			$this->error = $langs->trans('ParentNotFound');
			throw new Exception("", -1);
		}

		return $parent;
	}

	/**
	 * Call a method by giving an array instead of parameters
	 * It is use to call different addline, updateline, deleteline methods
	 * because of differents definition on each objects
	 * If a parameters does not exist in that function it is not transmitted
	 * @global type $langs
	 * @param type $instance
	 * @param type $method
	 * @param array $parameters
	 * @return type
	 */
	protected function _simple_method_call($instance, $method, array $parameters)
	{
		global $langs;

		if (!method_exists($instance, $method)) {
			$this->error = $langs->trans('MethodDoesNotExist', static::class, $method);
			return -1;
		}

		$finalParameters = [];

		$reflectionMethod = new ReflectionMethod($instance, $method);

		$arguments = $reflectionMethod->getParameters();

		/** @var ReflectionParameter $argument */
		foreach ($arguments as $argument) {
			$name = $argument->getName();

			$isOptional = $argument->isOptional();
			$isPassed = array_key_exists($name, $parameters);

			if (!$isOptional && !$isPassed) {
				$this->error = $langs->trans('ErrorFieldRequired', $name);
				return -1;
			}

			if ($isPassed) {
				$finalParameters[] = $parameters[$name];
				continue;
			}

			$finalParameters[] = $argument->getDefaultValue();
		}

		// return call_user_func_array([$instance, $method], $finalParameters);
		// $instance_class = is_string($instance) ? $instance : get_class($instance);
		$ret = call_user_func_array([$instance, $method], $finalParameters);
		if ($ret < 0) {
			$this->errors[] = $instance->errorsToString();
		}
		return $ret;
	}

	/**
	 * If ECOTAX_WORKFLOW_RECEPTION_CLASSIFY_CLOSED_INVOICE is set
	 * set receptions as billed when all total _without ecotaxes_ equals receptions total.
	 * @param FactureFournisseur $object
	 * @param User $user
	 * @param Translate $langs
	 * @param Conf $conf
	 * @return int
	 * @see InterfaceWorkflowManager::runTrigger() : case BILL_SUPPLIER_VALIDATE
	 */
	public function billSupplierValidate(FactureFournisseur $object, User $user, Translate $langs, Conf $conf)
	{
		if (isModEnabled("reception") && getDolGlobalInt('ECOTAX_WORKFLOW_RECEPTION_CLASSIFY_CLOSED_INVOICE')) {
			$ecotaxes = getEcotaxServiceIds($this->db);
			$total_without_ecotaxes = 0;
			foreach($object->lines as $line) {
				if (!in_array($line->fk_product, $ecotaxes)) {
					$total_without_ecotaxes += $line->total_ht;
				}
			}

			$object->fetchObjectLinked('', 'reception', $object->id, $object->element);
			if (!empty($object->linkedObjects)) {
				$totalonlinkedelements = 0;
				foreach ($object->linkedObjects['reception'] as $element) {
					if ($element->statut == Reception::STATUS_VALIDATED || $element->statut == Reception::STATUS_CLOSED) {
						$totalonlinkedelements += $element->total_ht;
					}
				}
				dol_syslog("Amount of linked reception = ".$totalonlinkedelements.", of invoice (without ecotaxes) = ".$total_without_ecotaxes.", egality is ".($totalonlinkedelements == $total_without_ecotaxes), LOG_DEBUG);
				if ((string) $totalonlinkedelements == (string) $total_without_ecotaxes) {
					foreach ($object->linkedObjects['reception'] as $element) {
						$ret = $element->setBilled();
						if ($ret < 0) {
							return $ret;
						}
					}
				}
			}
		}
		return 0;
	}

	/**
	 * If ECOTAX_SET_SUPPLIER_ORDER_FULL_RECEIVED is NOT set
	 * don't count ecotaxes to class a supplier order as 'fulle received'
	 * @param FactureFournisseur $object
	 * @param User $user
	 * @param Translate $langs
	 * @param Conf $conf
	 * @return int
	 * @see Reception::valid() ; Reception::getStatusDispatch()
	 */
	public function receptionValidate(Reception $object, User $user, Translate $langs, Conf $conf)
	{
		$error = 0;

		if (isModEnabled("fournisseur") && !getDolGlobalInt('ECOTAX_SET_SUPPLIER_ORDER_FULL_RECEIVED')) {
			$status = $this->_getStatusDispatch($object);
			if ($status < 0) {
				$error++;
			} else {
				$trigger_key = '';
				if ($status == CommandeFournisseur::STATUS_RECEIVED_COMPLETELY) {
					$ret = $object->commandeFournisseur->Livraison($user, dol_now(), 'tot', '');
					if ($ret < 0) {
						$error++;
						$object->errors = array_merge($object->errors, $object->commandeFournisseur->errors);
					}
				} else {
					$ret = $object->setStatut($status, $object->origin_id, 'commande_fournisseur', $trigger_key);
					if ($ret < 0) {
						$error++;
					}
				}
			}
		}

		if ($error) {
			return -$error;
		}

		return 0;
	}

	private function _getStatusDispatch(&$reception)
	{
		global $conf;

		require_once DOL_DOCUMENT_ROOT.'/fourn/class/fournisseur.commande.class.php';
		require_once DOL_DOCUMENT_ROOT.'/fourn/class/fournisseur.commande.dispatch.class.php';

		$status = CommandeFournisseur::STATUS_RECEIVED_PARTIALLY;

		if (!empty($reception->origin) && $reception->origin_id > 0 && ($reception->origin == 'order_supplier' || $reception->origin == 'commandeFournisseur')) {
			if (empty($reception->commandeFournisseur)) {
				$reception->fetch_origin();
				if (empty($reception->commandeFournisseur->lines)) {
					$res = $reception->commandeFournisseur->fetch_lines();
					if ($res < 0)	return $res;
				}
			}

			$qty_received = array();
			$qty_wished = array();

			$supplierorderdispatch = new CommandeFournisseurDispatch($this->db);
			$filter = array('t.fk_commande'=>$reception->origin_id);
			if (!empty($conf->global->SUPPLIER_ORDER_USE_DISPATCH_STATUS)) {
				$filter['t.status'] = 1; // Restrict to lines with status validated
			}

			$ret = $supplierorderdispatch->fetchAll('', '', 0, 0, $filter);
			if ($ret < 0) {
				$reception->error = $supplierorderdispatch->error;
				$reception->errors = $supplierorderdispatch->errors;
				return $ret;
			} else {

				$ecotaxes = getEcotaxServiceIds($this->db);

				// build array with quantity received by product in all supplier orders (origin)
				foreach ($supplierorderdispatch->lines as $dispatch_line) {
					if (in_array($dispatch_line->fk_product, $ecotaxes)) {
						continue;
					}
					if (array_key_exists($dispatch_line->fk_product, $qty_received)) {
						$qty_received[$dispatch_line->fk_product] += $dispatch_line->qty;
					} else {
						$qty_received[$dispatch_line->fk_product] = $dispatch_line->qty;
					}
				}

				// qty wished in order supplier (origin)
				foreach ($reception->commandeFournisseur->lines as $origin_line) {
					if (in_array($origin_line->fk_product, $ecotaxes)) {
						continue;
					}

					// exclude lines not qualified for reception
					if ((!getDolGlobalString('STOCK_SUPPORTS_SERVICES') && $origin_line->product_type > 0) || $origin_line->product_type > 1) {
						continue;
					}

					$qty_wished[$origin_line->fk_product] += $origin_line->qty;
				}

				// compare array
				$diff_array = array_diff_assoc($qty_received, $qty_wished); // Warning: $diff_array is done only on common keys.
				$keys_in_wished_not_in_received = array_diff(array_keys($qty_wished), array_keys($qty_received));
				$keys_in_received_not_in_wished = array_diff(array_keys($qty_received), array_keys($qty_wished));

				if (count($diff_array) == 0 && count($keys_in_wished_not_in_received) == 0 && count($keys_in_received_not_in_wished) == 0) { // no diff => mean everything is received
					$status = CommandeFournisseur::STATUS_RECEIVED_COMPLETELY;
				} elseif (!empty($conf->global->SUPPLIER_ORDER_MORE_THAN_WISHED)) {
					// set totally received if more products received than ordered
					$close = 0;

					if (count($diff_array) > 0) {
						// there are some difference between the two arrays
						// scan the array of results
						foreach ($diff_array as $key => $value) {
							// if the quantity delivered is greater or equal to ordered quantity
							if ($qty_received[$key] >= $qty_wished[$key]) {
								$close++;
							}
						}
					}

					if ($close == count($diff_array)) {
						// all the products are received equal or more than the ordered quantity
						$status = CommandeFournisseur::STATUS_RECEIVED_COMPLETELY;
					}
				}
			}
		}

		return $status;
	}
}
