<?php

/* Copyright (C) 2024 SuperAdmin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/**
 * \file    ecotax/lib/ecotax.lib.php
 * \ingroup ecotax
 * \brief   Library files with common functions for Ecotax
 */

/**
 * Prepare admin pages header
 *
 * @return array
 */
function ecotaxAdminPrepareHead()
{
	global $langs, $conf;

	// global $db;
	// $extrafields = new ExtraFields($db);
	// $extrafields->fetch_name_optionals_label('myobject');

	$langs->load("ecotax@ecotax");

	$h = 0;
	$head = array();

	$head[$h][0] = dol_buildpath("/ecotax/admin/setup.php", 1);
	$head[$h][1] = $langs->trans("Settings");
	$head[$h][2] = 'settings';
	$h++;

	/*
	  $head[$h][0] = dol_buildpath("/ecotax/admin/myobject_extrafields.php", 1);
	  $head[$h][1] = $langs->trans("ExtraFields");
	  $nbExtrafields = is_countable($extrafields->attributes['myobject']['label']) ? count($extrafields->attributes['myobject']['label']) : 0;
	  if ($nbExtrafields > 0) {
	  $head[$h][1] .= ' <span class="badge">' . $nbExtrafields . '</span>';
	  }
	  $head[$h][2] = 'myobject_extrafields';
	  $h++;
	 */

	$head[$h][0] = dol_buildpath("/ecotax/admin/dictionaries.php", 1);
	$head[$h][1] = $langs->trans("Dictionary");
	$head[$h][2] = 'dictionaries';
	$h++;


	$head[$h][0] = dol_buildpath("/ecotax/admin/about.php", 1);
    $head[$h][1] = $langs->trans("About") . " / " . $langs->trans("Support");
	$head[$h][2] = 'about';
	$h++;


	// Show more tabs from modules
	// Entries must be declared in modules descriptor with line
	//$this->tabs = array(
	//	'entity:+tabname:Title:@ecotax:/ecotax/mypage.php?id=__ID__'
	//); // to add new tab
	//$this->tabs = array(
	//	'entity:-tabname:Title:@ecotax:/ecotax/mypage.php?id=__ID__'
	//); // to remove a tab
	complete_head_from_modules($conf, $langs, null, $head, $h, 'ecotax@ecotax');

	complete_head_from_modules($conf, $langs, null, $head, $h, 'ecotax@ecotax', 'remove');

	return $head;
}

/**
 * Show ecotax table above totals
 * 
 * @param type $pdf
 * @param type $object
 * @param type $outputlangs
 * @return type
 */
function drawEcotaxTable(&$pdf, $object, $outputlangs) {
	$outputlangs->load('ecotax@ecotax');

	$ecotaxArray = findEcotaxTotals($object);

	$tab_top = $pdf->GetY();

	if (empty($ecotaxArray)) {
		return $tab_top;
	}

	$html = '
		<table>
			<tr>
				<td>' . $outputlangs->transnoentities("IncludingEcotaxAmount") . '</td>
				<td></td>
			</tr>';

	foreach ($ecotaxArray as $ecotaxcode => $ecotaxtotal) {
		$html .= '
			<tr>
				<td>' . $outputlangs->transnoentities("EcotaxLine") . ' ' . $ecotaxcode . ' HT</td>
				<td style="text-align: right;">' . price($ecotaxtotal) . '</td>
			</tr>';
	}

	$html .= '</table>';

	$posx = 120;
	$largcol = 80;

	$pdf->SetXY($posx, $tab_top);
	$tab_hl = 4 * (count($ecotaxArray) + 1);

	$pdf->writeHTMLCell($largcol, $tab_hl, '', '', $html, 0, 1);

	return ($tab_hl * 21);
}

/**
 * Function to determine ecotax total on object for each type
 * @return bool
 */
function findEcotaxTotals($object)
{
	global $db;

	$ecotaxArray = [];

	foreach ($object->lines as $line) {
		if (empty($line->fk_product)) {
			continue;
		}

		$sql = '
			SELECT code
			FROM ' . $db->prefix() . 'c_ecotax_type
			WHERE fk_service=' . $line->fk_product;

		$resql = $db->query($sql);

		if (!$resql) {
			continue;
		}

		$num = $db->num_rows($resql);

		if (empty($num)) {
			continue;
		}

		$obj = $db->fetch_object($resql);

		if (empty($ecotaxArray[$obj->code])) {
			$ecotaxArray[$obj->code] = $line->total_ht;
		} else {
			$ecotaxArray[$obj->code] += $line->total_ht;
		}
	}

	return $ecotaxArray;
}

/**
 * Get list of service IDs that are ecotax services.
 * Reads dictionary llx_c_ecotax_type.
 * @param	$db			database handler
 * @param	$active		select active dictionary lines. Values: null (default. Active + unactive), true (only actives), false (only unactive)
 * @return int|Array(int)	-1 if error, else Ecotax product IDs
 */
function getEcotaxServiceIds($db, $active = null)
{
	dol_syslog(__METHOD__.': $active='.$active, LOG_DEBUG);

	$sql = 'SELECT fk_service FROM '.$db->prefix().'c_ecotax_type';
	if ($active === false) {
		$sql.= ' WHERE active = 0';
	} elseif ($active === true) {
		$sql.= ' WHERE active = 1';
	}

	$resql = $db->query($sql);
	if (!$resql) {
		return -1;
	}
	$ecotax_service_ids = array();
	while ($obj = $db->fetch_object($resql)) {
		$ecotax_service_ids[] = (int) $obj->fk_service;
	}

	return $ecotax_service_ids;
}

/**
 * Function to determine either or not a line is a ecotax line
 * @param type $productId
 * @return bool
 */
function isEcotaxLine(&$db, $productId)
{
	$sql = '
		SELECT code
		FROM ' . $db->prefix() . 'c_ecotax_type
		WHERE fk_service=' . $productId;

	$resql = $db->query($sql);

	if (!$resql) {
		// $this->error = $db->lasterror();
		return false;
	}

	$num = $db->num_rows($resql);

	if (empty($num)) {
		return false;
	}

	return true;
}